/*F File_name: 
/*F	aCorr.c
/*
/* Calculates autocorrelation coefficients. */

/*
/*N Note: 
/*N     This program is altered to be compatiable with cascade.tcl program.
/*N     Amalgamate this program with cascade.tcl using "xmktclapp".  
/*N     The pathname specified in compilation is specific for skorpio machine.
/*

/* The output will be the autocorrelation coefficient covering the */
/* entire data set, with given gap size "k". The number of output values */
/* can be controlled by setting the gap stepsize, start, and end values. */
/* The mean of the input data and the maximum and minimum coefficients */
/* are output to stderr and not to the output file.                    */


/* Usage: autocorr infile outfile stepsize start end   */
/*
/* infile: the input file. Data should be ASCII real numbers, */
/*         one per line.				      */
/* stepsize: the amount the gap size "k" increases per iteration. Default */
/*           is 1.						*/
/* start: the value the gap size starts from. Default is 0.  */
/*        calculations).                                             */
/* end: the value the gap size ends at. Default is the length of the data. */
/* outfile: output data. The first column is the gap size, the second is */
/*          then autocorrelation coefficient for that gap size.          */

#include <stdio.h>
#include <stdlib.h>
#include "synTraff.h"

int ET_COMMAND_mkAc(ET_TCLARGS)
{
   FILE *infile, *outfile;
   double mean = 0.0;
   int counter = 0;
   int stepsize, start, end;
   float current;
   float *buffer,*p;
   double numerator,denominator,t;
   int g, i;
   double max_coeff = -1.0, min_coeff = 1.0;
   double coeff;

   if (argc < 6)
   {
      Et_ResultF(interp, "Usage: mkAc infile outfile stepsize start end\n");
      return TCL_ERROR;
   }

   if ((infile = fopen(argv[1],"r")) == NULL)
   {
      Et_ResultF(interp, "Unable to open input file %s.\n",argv[1]);
      return TCL_ERROR;
   }
   outfile = fopen(argv[2],"w");

   stepsize = atoi(argv[3]);
   start    = atoi(argv[4]);
   end      = atoi(argv[5]);

   /* Find mean */
   while (fscanf(infile,"%f",&current) != EOF)
   {
      mean += (double) current;
      counter++;
   }

   mean /= counter;

   /* Number of values known - allocate storage. */

   if ((buffer = (float *) malloc(counter*sizeof(float))) == NULL)
   {
      Et_ResultF(interp, "Memory allocation error.\n");
      return TCL_ERROR;
   }

   p = buffer;
 
   if (end > (counter - 1))
      end = (counter - 1);

   
   /* Read values into array. */
   rewind(infile);

   while (fscanf(infile,"%f",p++) != EOF);


   /* Find autocorrelation coefficients. */ 
   for (g = start; g < end; g += stepsize)
   {

      p = buffer + g;

      numerator = denominator = 0.0;

      for (i = g; i < counter; i++,p++)
      {
         numerator += ((double) (*p) - mean)*((double) (*(p-g)) - mean);
         t = (double) (*p) - mean;
         denominator += t*t;
      }


      /* This part handles the case when numerator and denominator are */
      /* equal to zero (in which case the autocorrelation coeff. is    */
      /* zero). The "epsilon" value (.00001) is for handling floating  */
      /* point errors.                                                 */
      if ((denominator < 0.00001) && (numerator < 0.00001) && (numerator > -0.00001))
         coeff = 1.0;
      else
         coeff = numerator / denominator;

      fprintf(outfile,"%d %10.6lf\n",g,coeff);

   }

   fclose(infile);
   fclose(outfile);

   return TCL_OK;

} 



