/*F File_name: 
/*F	scale.c
/*
/*P Purpose:
/*P 	The purpose of this program is to take a zero mean
/*P 	synthetically generated long-range dependent time series
/*P     and do a mathematical transformation of all data points
/*P     by multiplying by a numeric constant. This will preserve
/*P     the self-similarity, but convert to a new range of numbers
/*P     (i.e., changing the spread from the smallest to the largest)
/*P     The mean of the time series will scale accordingly, though
/*P     you only really notice this if the mean is not zero initially.
/*P     The variance will also scale, but not the peak to mean ratio.
/*
/*U Usage:
/*U     cc -o scale scale.c
/*U     scale value < input_file > output_file
/*
/*E Examples:
/*E     scale 10.0 < H5.original.dat > H5.scale10.dat
/*E       (this changes range [-1,1] into range [-10,10]).
/*E     cat H9.dat | scale 10.0 | translate 10.0 > H9.scale10.translate10.dat
/*E         (this converts to non-negative numbers, with mean 10)
/*E         (i.e., from range [-1,+1] to range [0,20])
/*
/*A Author:
/*A  	Carey Williamson (and modified by Ram Balakrishnan, February 1999)
/*
/*D Date:
/*D 	July 1996
/*
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

main(argc, argv)
int argc;
char **argv;
{
  int i;
  float value, num;

  if(argc < 2)
  {
    printf("Usage: scale value < input_file > output_file\n");
    exit(-1);
  }

  value = atof(argv[1]);

  while( (i = scanf("%f\n", &num)) == 1 )
    printf("%f\n", num * value);

  return (0);
}










