
##########################################################
#                                                        #
#                 Synthetic Traffic Generator            #
#                                                        #
#                    Ver1: February 1999                 #
#                    Ver2: March    1999                 #
#                    Ver3: March    1999                 #
#                                                        #
#                     Generation Module                  #
#                                                        #
##########################################################

# This program uses several procedures written in C.
# You can create an executable version of this script by amalgamating this
#  with the following C programs, using "xmktclapp" and compiling the output
#  as specified under usage.

# C procedures used by this script.
#   
#   MeanStd.c  // To determine the mean and STD of a time series.
#   mult.c     // To refine a time seires down to lower time scales.
#   aCorr.c    // To create autocorreation points of a time series.
#   rs.c       // To create RS points of a time series
#   variance.c // To create variance-time of a time series.
#   buckets.c  // To create marginal distribution plot
#   sprinkle.c // To sprinkle a time seires down to lower time scales.

# Requirements: synTraff requires the following executable program  
#                        in the working directory
#        hosking
#        fancyshift
#        scale
#        tes_ref

# Usage: Specific for Skorpio machine
#   gcc -o synTraff MeanStd.c mult.c sprinkle.c aCorr.c rs.c variance.c buckets.c synTraff.c -L/sw/arch/lib -ltk8.0 -ltcl8.0 -lX11 -lm -ldl
#   gcc -static -o synTraff MeanStd.c mult.c sprinkle.c aCorr.c rs.c variance.c buckets.c synTraff.c -L/sw/arch/lib -ltk8.0 -ltcl8.0 -lX11 -lm -lsocket -lnsl -I/sw/ind/include
#   synTraff &

#--------------------------------------------------------------------------


# Set window title
wm title . "Synthetic Traffic Generation Kit"

. config -bg #cccccc

#--------------------------------------------------------------------------
#        *** Generation Module ***

frame .gn -relief groove -borderwidth 3 -bg #cccccc
pack .gn -side top


# Entry for the coarse grain traffic file.

label .gn.inFlL -text "Output file name: " -bg #cccccc -fg black
grid .gn.inFlL -row 1 -column 1

entry .gn.inFl -width 20 -textvariable orgFl -xscrollcommand cpFlN \
	-bg #cccccc -fg black
grid .gn.inFl -row 1 -column 2 

proc cpFlN {x1 x2} {
    global orgFl inFileName sprInFlNm tesInFlNm
    set inFileName "$orgFl"
    set sprInFlNm "$orgFl"
    set tesInFlNm "$orgFl"
} 


# Hurst Value

frame .gn.hstFm -relief groove -borderwidth 3 -bg #cccccc 
grid .gn.hstFm -row 2 -column 1

scale .gn.hstFm.sl -from 0.5 -to 1.0 -bigincrement 0.1 -command HslP \
      -orient horizontal -length 200 -variable Hst  \
      -resolution .01 -showvalue 0 -bg #cccccc -fg black

.gn.hstFm.sl set 0.8 
label .gn.hstFm.slVal -text "Hurst: 0.80" -bg #cccccc -fg black

proc HslP {Ascl} {
	.gn.hstFm.slVal configure -text "Hurst: $Ascl"
}

label .gn.hstFm.xAxVal -text "0.5  0.6  0.7  0.8  0.9  1.0" \
	-bg #cccccc -fg black

pack .gn.hstFm.slVal .gn.hstFm.sl .gn.hstFm.xAxVal -side top



# No.of Data points

frame .gn.dSlFm -relief groove -borderwidth 3 -bg #cccccc
grid .gn.dSlFm -row 2 -column 2

scale .gn.dSlFm.rng -from 1 -to 38 -bigincrement 10 -command DscaleP \
      -orient horizontal -length 230 -variable DIscl -showvalue 0 \
      -bg #cccccc -fg black

.gn.dSlFm.rng set 11
set Dscl 10

proc DscaleP {Ascl} {
    global Dscl
    if {$Ascl <= 10} {
	set Dscl [expr $Ascl - 1]
	.gn.dSlFm.rngVal configure -text "Data Points:    $Dscl"
    } elseif {$Ascl <= 20} {
	set Dscl [expr ($Ascl - 10) * 10]
	.gn.dSlFm.rngVal configure -text "Data Points:   $Dscl"
    } elseif {$Ascl <= 29} {
	set Dscl [expr ($Ascl - 19) * 100]
	.gn.dSlFm.rngVal configure -text "Data Points:  $Dscl"
    } else {
	set Dscl [expr ($Ascl - 28) * 1000]
	.gn.dSlFm.rngVal configure -text "Data Points: $Dscl"
    }    
}

label .gn.dSlFm.rngVal -text "Data Points:    10" -bg #cccccc -fg black

label .gn.dSlFm.xAxVal -text "          10      100    1000   10000" \
	-bg #cccccc -fg black

pack .gn.dSlFm.rngVal .gn.dSlFm.rng .gn.dSlFm.xAxVal -side top


 

# Variance parameter

frame .gn.varFm -relief groove -borderwidth 3 -bg #cccccc
grid .gn.varFm -row 3 -column 1

scale .gn.varFm.sl -from 0.0 -to 1.0 -bigincrement 0.1 -command VslP \
      -orient horizontal -length 200 -variable Var  \
      -resolution .01 -showvalue 0 -bg #cccccc -fg black

.gn.varFm.sl set 0.5
label .gn.varFm.slVal -text "Variability: 0.50" -bg #cccccc -fg black

proc VslP {Ascl} {
	.gn.varFm.slVal configure -text "Variability: $Ascl"
}

label .gn.varFm.xAxVal -text "0.0  0.2  0.4  0.6  0.8  1.0" \
	-bg #cccccc -fg black

pack .gn.varFm.slVal .gn.varFm.sl .gn.varFm.xAxVal -side top



# Approximate mean of the input

frame .gn.mnFm -relief groove -borderwidth 3 -bg #cccccc
grid .gn.mnFm -row 3 -column 2 -ipady 21 -ipadx 9


entry .gn.mnFm.mn -width 15 -textvariable orgMn  -bg #cccccc -fg black

label .gn.mnFm.enVal -text "Approximate Mean:  " -bg #cccccc -fg black
set orgMn "100000"

pack .gn.mnFm.enVal .gn.mnFm.mn -side left 


# Create Action Buttons

button .gn.gen -text "Generate" -command genP -bg #cccccc -fg black
grid .gn.gen -row 4 -column 1

proc genP {} {
    global Hst Dscl Var orgMn orgFl
    set d [expr $Hst-0.5]
    if {$d == 0.5} { set d 0.49 }
    set den [expr 1-$Var]
    exec hosking $d $Dscl | fancyshift $den | scale $orgMn > $orgFl
}

button .gn.rst -text "Reset" -command rstP -bg #cccccc -fg black
grid .gn.rst -row 4 -column 2

proc rstP {} {
    global orgMn
    .gn.inFl delete 0 end
    .gn.hstFm.sl set 0.8 
    .gn.dSlFm.rng set 11
    .gn.varFm.sl set 0.5
    set orgMn "100000"
}



#################################################################
#                                                               #
#   Begining of support library for Notebook function           #
#                                                               #
#################################################################

#
# Create a new notebook widget
#
proc Notebook:create {w args} {
  global Notebook
  set Notebook($w,width) 370
  set Notebook($w,height) 225
  set Notebook($w,pages) {}
  set Notebook($w,top) 0
  set Notebook($w,pad) 0
  set Notebook($w,fg,on) black
  set Notebook($w,fg,off) grey50
  canvas $w -bd 0 -highlightthickness 0 -takefocus 0
  set Notebook($w,bg) [$w cget -bg]
  bind $w <1> "Notebook:click $w %x %y"
  bind $w <Configure> "Notebook:scheduleExpand $w"
  eval Notebook:config $w $args
}

#
# Change configuration options for the notebook widget
#
proc Notebook:config {w args} {
  global Notebook
  foreach {tag value} $args {
    switch -- $tag {
      -width {
        set Notebook($w,width) $value
      }
      -height {
        set Notebook($w,height) $value
      }
      -pages {
        set Notebook($w,pages) $value
      }
      -pad {
        set Notebook($w,pad) $value
      }
      -bg {
        set Notebook($w,bg) $value
      }
      -fg {
        set Notebook($w,fg,on) $value
      }
      -disabledforeground {
        set Notebook($w,fg,off) $value
      }
    }
  }

  #
  # After getting new configuration values, reconstruct the widget
  #
  $w delete all
  set Notebook($w,x1) $Notebook($w,pad)
  set Notebook($w,x2) [expr $Notebook($w,x1)+2]
  set Notebook($w,x3) [expr $Notebook($w,x2)+$Notebook($w,width)]
  set Notebook($w,x4) [expr $Notebook($w,x3)+2]
  set Notebook($w,y1) [expr $Notebook($w,pad)+2]
  set Notebook($w,y2) [expr $Notebook($w,y1)+2]
  set Notebook($w,y5) [expr $Notebook($w,y1)+30]
  set Notebook($w,y6) [expr $Notebook($w,y5)+2]
  set Notebook($w,y3) [expr $Notebook($w,y6)+$Notebook($w,height)]
  set Notebook($w,y4) [expr $Notebook($w,y3)+2]
  set x $Notebook($w,x1)
  set cnt 0
  set y7 [expr $Notebook($w,y1)+10]
  foreach p $Notebook($w,pages) {
    set Notebook($w,p$cnt,x5) $x
    set id [$w create text 0 0 -text $p -anchor nw -tags "p$cnt t$cnt"]
    set bbox [$w bbox $id]
    set width [lindex $bbox 2]
    $w move $id [expr $x+10] $y7
    $w create line \
       $x $Notebook($w,y5)\
       $x $Notebook($w,y2) \
       [expr $x+2] $Notebook($w,y1) \
       [expr $x+$width+16] $Notebook($w,y1) \
       -width 2 -fill white -tags p$cnt
    $w create line \
       [expr $x+$width+16] $Notebook($w,y1) \
       [expr $x+$width+18] $Notebook($w,y2) \
       [expr $x+$width+18] $Notebook($w,y5) \
       -width 2 -fill black -tags p$cnt
    set x [expr $x+$width+20]
    set Notebook($w,p$cnt,x6) [expr $x-2]
    if {![winfo exists $w.f$cnt]} {
      frame $w.f$cnt -bd 0
    }
    $w.f$cnt config -bg $Notebook($w,bg)
    place $w.f$cnt -x $Notebook($w,x2) -y $Notebook($w,y6) \
      -width $Notebook($w,width) -height $Notebook($w,height)
    incr cnt
  }
  $w create line \
     $Notebook($w,x1) [expr $Notebook($w,y5)-2] \
     $Notebook($w,x1) $Notebook($w,y3) \
     -width 2 -fill white
  $w create line \
     $Notebook($w,x1) $Notebook($w,y3) \
     $Notebook($w,x2) $Notebook($w,y4) \
     $Notebook($w,x3) $Notebook($w,y4) \
     $Notebook($w,x4) $Notebook($w,y3) \
     $Notebook($w,x4) $Notebook($w,y6) \
     $Notebook($w,x3) $Notebook($w,y5) \
     -width 2 -fill black
  $w config -width [expr $Notebook($w,x4)+$Notebook($w,pad)] \
            -height [expr $Notebook($w,y4)+$Notebook($w,pad)] \
            -bg $Notebook($w,bg)
  set top $Notebook($w,top)
  set Notebook($w,top) -1
  Notebook:raise.page $w $top
}

#
# This routine is called whenever the mouse-button is pressed over
# the notebook.  It determines if any page should be raised and raises
# that page.
#
proc Notebook:click {w x y} {
  global Notebook
  if {$y<$Notebook($w,y1) || $y>$Notebook($w,y6)} return
  set N [llength $Notebook($w,pages)]
  for {set i 0} {$i<$N} {incr i} {
    if {$x>=$Notebook($w,p$i,x5) && $x<=$Notebook($w,p$i,x6)} {
      Notebook:raise.page $w $i
      break
    }
  }
}

#
# For internal use only.  This procedure raised the n-th page of
# the notebook
#
proc Notebook:raise.page {w n} {
  global Notebook
  if {$n<0 || $n>=[llength $Notebook($w,pages)]} return
  set top $Notebook($w,top)
  if {$top>=0 && $top<[llength $Notebook($w,pages)]} {
    $w move p$top 0 2
  }
  $w move p$n 0 -2
  $w delete topline
  if {$n>0} {
    $w create line \
       $Notebook($w,x1) $Notebook($w,y6) \
       $Notebook($w,x2) $Notebook($w,y5) \
       $Notebook($w,p$n,x5) $Notebook($w,y5) \
       $Notebook($w,p$n,x5) [expr $Notebook($w,y5)-2] \
       -width 2 -fill white -tags topline
  }
  $w create line \
    $Notebook($w,p$n,x6) [expr $Notebook($w,y5)-2] \
    $Notebook($w,p$n,x6) $Notebook($w,y5) \
    -width 2 -fill white -tags topline
  $w create line \
    $Notebook($w,p$n,x6) $Notebook($w,y5) \
    $Notebook($w,x3) $Notebook($w,y5) \
    -width 2 -fill white -tags topline
  set Notebook($w,top) $n
  raise $w.f$n
}

#
# Change the page-specific configuration options for the notebook
#
proc Notebook:pageconfig {w name args} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i<0} return
  foreach {tag value} $args {
    switch -- $tag {
      -state {
        if {"$value"=="disabled"} {
          $w itemconfig t$i -fg $Notebook($w,fg,off)
        } else {
          $w itemconfig t$i -fg $Notebook($w,fg,on)
        }
      }
      -onexit {
        set Notebook($w,p$i,onexit) $value
      }
    }
  }
}

#
# This procedure raises a notebook page given its name.  But first
# we check the "onexit" procedure for the current page (if any) and
# if it returns false, we don't allow the raise to proceed.
#
proc Notebook:raise {w name} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i<0} return
  if {[info exists Notebook($w,p$i,onexit)]} {
    set onexit $Notebook($w,p$i,onexit)
    if {"$onexit"!="" && [eval uplevel #0 $onexit]!=0} {
      Notebook:raise.page $w $i
    }
  } else {
    Notebook:raise.page $w $i
  }
}

#
# Return the frame associated with a given page of the notebook.
#
proc Notebook:frame {w name} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i>=0} {
    return $w.f$i
  } else {
    return {}
  }
}

#
# Try to resize the notebook to the next time we become idle.
#
proc Notebook:scheduleExpand w {
  global Notebook
  if {[info exists Notebook($w,expand)]} return
  set Notebook($w,expand) 1
  after idle "Notebook:expand $w"
}

#
# Resize the notebook to fit inside its containing widget.
#
proc Notebook:expand w {
  global Notebook
  set wi [expr [winfo width $w]-($Notebook($w,pad)*2+4)]
  set hi [expr [winfo height $w]-($Notebook($w,pad)*2+36)]
  Notebook:config $w -width $wi -height $hi
  catch {unset Notebook($w,expand)}
}


#################################################################
#                                                               #
#   End of support library for Notebook function                #
#                                                               #
#################################################################





##########################################################
#                                                        #
#                     Refinement Module                  #
#                                                        #
##########################################################

Notebook:create .ref -pages {"Sprinkle" "Cascade" "TES"} -bg #cccccc
pack .ref -side top

#     ### Cascade ###    #

set casPg [Notebook:frame .ref "Cascade"]

frame $casPg.cas -relief groove -borderwidth 3 -bg #cccccc
pack $casPg.cas -side top

# Entry for the coarse grain traffic file.

label $casPg.cas.inFileMsg -text "Coarse traffic (Input)  file name: " \
	-bg #cccccc -fg black
grid $casPg.cas.inFileMsg -row 1 -column 1

entry $casPg.cas.inFile -width 20 -textvariable inFileName \
	-xscrollcommand mkOpFnP -bg #cccccc -fg black
grid $casPg.cas.inFile -row 1 -column 2 -columnspan 2

proc mkOpFnP {x1 x2} {
    global inFileName outFileName DisFn
    set outFileName "$inFileName.cas"
    set DisFn $inFileName
} 

# Entry for the fine grain traffic file.

label $casPg.cas.outFileMsg -text "Refined traffic (Output) file name: " \
	-bg #cccccc -fg black
grid $casPg.cas.outFileMsg -row 2 -column 1

entry $casPg.cas.outFile -width 20 -textvariable outFileName \
	-bg #cccccc -fg black
grid $casPg.cas.outFile -row 2 -column 2 -columnspan 2

# Instruction about mean and STD
frame $casPg.cas.ms -relief groove -borderwidth 1 -bg #cccccc 
grid $casPg.cas.ms -row 3 -column 1 -columnspan 3 -ipadx 3 -pady 10

label $casPg.cas.ms.ins1 -bg #cccccc -fg black \
	-text "For actual mean and STD of i/p traffic, hit \"Calculate\"." 
label $casPg.cas.ms.ins2 -bg #cccccc -fg black \
	-text "To purposefully alter the values, enter them." 

grid $casPg.cas.ms.ins1 -row 1 -column 1 -columnspan 3
grid $casPg.cas.ms.ins2 -row 2 -column 1 -columnspan 3 -pady 2

# Mean of the coarse traffic

label $casPg.cas.ms.mMsg -text "Mean of the coarse traffic: " \
	-bg #cccccc -fg black
grid $casPg.cas.ms.mMsg -row 3 -column 1

entry $casPg.cas.ms.mean -width 15 -textvariable meanVal -bg #cccccc -fg black
grid $casPg.cas.ms.mean -row 3 -column 2

button $casPg.cas.ms.bCal -text "Calculate" -command {M_S_Calc $inFileName} \
	-bg #cccccc -fg black
grid $casPg.cas.ms.bCal -row 3 -column 3 -rowspan 2

proc M_S_Calc {file} {
    global meanVal STDVal
    set inter [MeanStd $file]
    set tmp [split $inter]
    set meanVal [lindex $tmp 0]
    set STDVal [lindex $tmp 1]
}

# STD of the coarse traffic

label $casPg.cas.ms.sMsg -text "STD of the coarse traffic: " \
	-bg #cccccc -fg black
grid $casPg.cas.ms.sMsg -row 4 -column 1

entry $casPg.cas.ms.std -width 15 -textvariable STDVal -bg #cccccc -fg black
grid $casPg.cas.ms.std -row 4 -column 2


# sub-interval range

frame $casPg.cas.slFm -relief groove -borderwidth 3 -bg #cccccc
grid $casPg.cas.slFm -row 4 -column 1

scale $casPg.cas.slFm.rng -from 1 -to 29 -bigincrement 10 -command scaleP \
      -orient horizontal -length 200 -showvalue 0 \
      -bg #cccccc -fg black
$casPg.cas.slFm.rng set 11
set scl 10

proc scaleP {Ascl} {
    global scl casPg
    if {$Ascl <= 10} {
	set scl [expr $Ascl - 1]
	$casPg.cas.slFm.rngVal configure -text "Subinterval:   $scl"
    } elseif {$Ascl <= 20} {
	set scl [expr ($Ascl - 10) * 10]
	$casPg.cas.slFm.rngVal configure -text "Subinterval:  $scl"
    } else {
	set scl [expr ($Ascl - 19) * 100]
	$casPg.cas.slFm.rngVal configure -text "Subinterval: $scl"
    }    
}

label $casPg.cas.slFm.rngVal -text "Subinterval:   10" -bg #cccccc -fg black

label $casPg.cas.slFm.xAxVal -text "          10     100     1000" \
	-bg #cccccc -fg black

pack $casPg.cas.slFm.rngVal $casPg.cas.slFm.rng $casPg.cas.slFm.xAxVal \
	-side top
 
# Create Action Buttons

button $casPg.cas.refine -text "Cascade" -command refineP -bg #cccccc -fg black
grid $casPg.cas.refine -row 4 -column 2

proc refineP {} {
    global scl meanVal STDVal inFileName outFileName DisFn
    refine $scl [expr $meanVal/$scl] [expr $STDVal/($scl*1.5)] $inFileName \
	    $outFileName
    set DisFn $outFileName
}

button $casPg.cas.reset -text "Reset" -command resetP -bg #cccccc -fg black
grid $casPg.cas.reset -row 4 -column 3

proc resetP {} {
    global casPg
    $casPg.cas.inFile delete 0 end
    $casPg.cas.outFile delete 0 end
    $casPg.cas.ms.mean delete 0 end
    $casPg.cas.ms.std delete 0 end
    $casPg.cas.slFm.rng set 11
}


#*************************#
#     ### Sprinkle ###    #
#*************************#

set sprPg [Notebook:frame .ref "Sprinkle"]

frame $sprPg.spr -relief groove -borderwidth 3 -bg #cccccc
pack $sprPg.spr -side top

# Entry for the coarse grain traffic file.

label $sprPg.spr.inFileMsg -text "Coarse traffic (Input)  file name: " \
	-bg #cccccc -fg black
grid $sprPg.spr.inFileMsg -row 1 -column 1

entry $sprPg.spr.inFile -width 20 -textvariable sprInFlNm \
        -xscrollcommand sprmkOpFnP -bg #cccccc -fg black
grid $sprPg.spr.inFile -row 1 -column 2 -columnspan 2

proc sprmkOpFnP {x1 x2} {
    global sprInFlNm sprOtFlNm DisFn
    set sprOtFlNm "$sprInFlNm.spr"
    set DisFn $sprInFlNm
} 

# Entry for the fine grain traffic file.

label $sprPg.spr.outFileMsg -text "Refined traffic (Output) file name: " \
	-bg #cccccc -fg black
grid $sprPg.spr.outFileMsg -row 2 -column 1

entry $sprPg.spr.outFile -width 20 -textvariable sprOtFlNm -bg #cccccc \
	-fg black
grid $sprPg.spr.outFile -row 2 -column 2 -columnspan 2

# Higher threshold

frame $sprPg.spr.varFm -relief groove -borderwidth 3 -bg #cccccc
grid $sprPg.spr.varFm -row 3 -column 1 -columnspan 3 -ipadx 3 -pady 18

scale $sprPg.spr.varFm.sl -from 0.0 -to 1.0 -bigincrement 0.1 -command sprHTP \
      -orient horizontal -length 200 -variable HT  \
      -resolution .01 -showvalue 0 -bg #cccccc -fg black

$sprPg.spr.varFm.sl set 1.0
label $sprPg.spr.varFm.slVal -text "HT: 1.00" -bg #cccccc -fg black

proc sprHTP {Ascl} {
    global sprPg
    $sprPg.spr.varFm.slVal configure -text "HT: $Ascl"
}

label $sprPg.spr.varFm.xAxVal -text "0.0  0.2  0.4  0.6  0.8  1.0" \
	-bg #cccccc -fg black

pack $sprPg.spr.varFm.slVal $sprPg.spr.varFm.sl $sprPg.spr.varFm.xAxVal \
	-side top


# sub-interval range

frame $sprPg.spr.slFm -relief groove -borderwidth 3 -bg #cccccc
grid $sprPg.spr.slFm -row 4 -column 1

scale $sprPg.spr.slFm.rng -from 1 -to 29 -bigincrement 10 -command sprSlP \
      -orient horizontal -length 200 -showvalue 0 \
      -bg #cccccc -fg black
$sprPg.spr.slFm.rng set 11
set sprSl 10

proc sprSlP {Ascl} {
    global sprSl sprPg
    if {$Ascl <= 10} {
	set sprSl [expr $Ascl - 1]
	$sprPg.spr.slFm.rngVal configure -text "Subinterval:   $sprSl"
    } elseif {$Ascl <= 20} {
	set sprSl [expr ($Ascl - 10) * 10]
	$sprPg.spr.slFm.rngVal configure -text "Subinterval:  $sprSl"
    } else {
	set sprSl [expr ($Ascl - 19) * 100]
	$sprPg.spr.slFm.rngVal configure -text "Subinterval: $sprSl"
    }    
}

label $sprPg.spr.slFm.rngVal -text "Subinterval:   10" -bg #cccccc -fg black

label $sprPg.spr.slFm.xAxVal -text "          10     100     1000" \
	-bg #cccccc -fg black

pack $sprPg.spr.slFm.rngVal $sprPg.spr.slFm.rng $sprPg.spr.slFm.xAxVal \
	-side top
 
# Create Action Buttons

button $sprPg.spr.refine -text "Sprinkle" -command sprinkleP \
	-bg #cccccc -fg black
grid $sprPg.spr.refine -row 4 -column 2

proc sprinkleP {} {
    global sprSl sprInFlNm sprOtFlNm DisFn HT
    sprinkle $HT $sprSl $sprInFlNm $sprOtFlNm
    set DisFn $sprOtFlNm
}

button $sprPg.spr.reset -text "Reset" -command sprRstP -bg #cccccc -fg black
grid $sprPg.spr.reset -row 4 -column 3

proc sprRstP {} {
    global sprPg
    $sprPg.spr.inFile delete 0 end
    $sprPg.spr.outFile delete 0 end
    $sprPg.spr.slFm.rng set 11
}


#********************#
#     ### TES ###    #
#********************#

set tesPg [Notebook:frame .ref "TES"]

frame $tesPg.tes -relief groove -borderwidth 3 -bg #cccccc
pack $tesPg.tes -side top

# Entry for the coarse grain traffic file.

label $tesPg.tes.inFileMsg -text "Coarse traffic (Input)  file name: " \
	-bg #cccccc -fg black
grid $tesPg.tes.inFileMsg -row 1 -column 1

entry $tesPg.tes.inFile -width 20 -textvariable tesInFlNm \
        -xscrollcommand tesmkOpFnP -bg #cccccc -fg black
grid $tesPg.tes.inFile -row 1 -column 2 -columnspan 2

proc tesmkOpFnP {x1 x2} {
    global tesInFlNm tesOtFlNm DisFn
    set tesOtFlNm "$tesInFlNm.tes"
    set DisFn $tesInFlNm
} 

# Entry for the fine grain traffic file.

label $tesPg.tes.outFileMsg -text "Refined traffic (Output) file name: " \
	-bg #cccccc -fg black
grid $tesPg.tes.outFileMsg -row 2 -column 1

entry $tesPg.tes.outFile -width 20 -textvariable tesOtFlNm -bg #cccccc \
	-fg black
grid $tesPg.tes.outFile -row 2 -column 2 -columnspan 2

# Hurst Value

frame $tesPg.tes.hstFm -relief groove -borderwidth 3 -bg #cccccc 
grid $tesPg.tes.hstFm -row 3 -column 1 -columnspan 3 -ipadx 3 -pady 18

scale $tesPg.tes.hstFm.sl -from 0.5 -to 1.0 -bigincrement 0.1 \
	-command tesHslP -orient horizontal -length 200 -variable tesHst  \
	-resolution .01 -showvalue 0 -bg #cccccc -fg black

$tesPg.tes.hstFm.sl set 0.8 
label $tesPg.tes.hstFm.slVal -text "Hurst: 0.80" -bg #cccccc -fg black

proc tesHslP {Ascl} {
    global tesPg
    $tesPg.tes.hstFm.slVal configure -text "Hurst: $Ascl"
}

label $tesPg.tes.hstFm.xAxVal -text "0.5  0.6  0.7  0.8  0.9  1.0" \
	-bg #cccccc -fg black

pack $tesPg.tes.hstFm.slVal $tesPg.tes.hstFm.sl $tesPg.tes.hstFm.xAxVal \
	-side top


# sub-interval range

frame $tesPg.tes.slFm -relief groove -borderwidth 3 -bg #cccccc
grid $tesPg.tes.slFm -row 4 -column 1

scale $tesPg.tes.slFm.rng -from 1 -to 29 -bigincrement 10 -command tesSlP \
      -orient horizontal -length 200 -showvalue 0 \
      -bg #cccccc -fg black
$tesPg.tes.slFm.rng set 11
set tesSl 10

proc tesSlP {Ascl} {
    global tesSl tesPg
    if {$Ascl <= 10} {
	set tesSl [expr $Ascl - 1]
	$tesPg.tes.slFm.rngVal configure -text "Subinterval:   $tesSl"
    } elseif {$Ascl <= 20} {
	set tesSl [expr ($Ascl - 10) * 10]
	$tesPg.tes.slFm.rngVal configure -text "Subinterval:  $tesSl"
    } else {
	set tesSl [expr ($Ascl - 19) * 100]
	$tesPg.tes.slFm.rngVal configure -text "Subinterval: $tesSl"
    }    
}

label $tesPg.tes.slFm.rngVal -text "Subinterval:   10" -bg #cccccc -fg black

label $tesPg.tes.slFm.xAxVal -text "          10     100     1000" \
	-bg #cccccc -fg black

pack $tesPg.tes.slFm.rngVal $tesPg.tes.slFm.rng $tesPg.tes.slFm.xAxVal \
	-side top
 
# Create Action Buttons

button $tesPg.tes.refine -text "TES Refine" -command tesRefP \
	-bg #cccccc -fg black
grid $tesPg.tes.refine -row 4 -column 2

proc tesRefP {} {
    global tesSl tesInFlNm tesOtFlNm DisFn tesHst
    exec tes_ref $tesSl $tesHst < $tesInFlNm > $tesOtFlNm
    set DisFn $tesOtFlNm
}

button $tesPg.tes.reset -text "Reset" -command tesRstP -bg #cccccc -fg black
grid $tesPg.tes.reset -row 4 -column 3

proc tesRstP {} {
    global tesPg
    $tesPg.tes.inFile delete 0 end
    $tesPg.tes.outFile delete 0 end
    $tesPg.tes.slFm.rng set 11
    $tesPg.tes.hstFm.sl set 0.8
}


##########################################################
#                                                        #
#                       Display Module                   #
#                                                        #
##########################################################



frame .dis -relief groove -borderwidth 3 -bg #cccccc
pack .dis -side left

# Entry for the filename

label .dis.fnL -text "Data file name: " -bg #cccccc -fg black
grid .dis.fnL -row 1 -column 1

entry .dis.fn -width 20 -textvariable DisFn -bg #cccccc -fg black
grid .dis.fn -row 1 -column 2 

# Entry for the Tile

label .dis.tlL -text "Title (Opt.):" -bg #cccccc -fg black
grid .dis.tlL -row 2 -column 1

entry .dis.tl -width 20 -textvariable DisTl  -bg #cccccc -fg black
grid .dis.tl -row 2 -column 2 

# Entry for Profile X limit

label .dis.pXlL -text "Profile X Limit (Opt.): " -bg #cccccc -fg black
grid .dis.pXlL -row 3 -column 1

entry .dis.pXl -width 20 -textvariable PdisXl  -bg #cccccc -fg black
grid .dis.pXl -row 3 -column 2 

# Entry for Autocorrelation X limit

label .dis.aLgL -text "AC lags (Opt.): " -bg #cccccc -fg black
grid .dis.aLgL -row 4 -column 1

entry .dis.aLg -width 20 -textvariable Alag -bg #cccccc -fg black 
grid .dis.aLg -row 4 -column 2 

# Entry for Distribution limit

label .dis.mDisL -text "Distribution Limit: " -bg #cccccc -fg black
grid .dis.mDisL -row 5 -column 1

entry .dis.mDis -width 20 -textvariable MdisL -bg #cccccc -fg black 
grid .dis.mDis -row 5 -column 2 

# No.of distributions 

frame .dis.mdF -relief groove -borderwidth 3 -bg #cccccc
grid .dis.mdF -row 6 -column 1 -columnspan 2

scale .dis.mdF.rng -from 1 -to 29 -bigincrement 10 -command mdSlP \
      -orient horizontal -length 200 -variable mdIscl -showvalue 0 \
      -bg #cccccc -fg black
.dis.mdF.rng set 20
set mdSl 100

proc mdSlP {Ascl} {
    global mdSl
    if {$Ascl <= 10} {
	set mdSl [expr $Ascl - 1]
	.dis.mdF.rngVal configure -text "Distributions:   $mdSl"
    } elseif {$Ascl <= 20} {
	set mdSl [expr ($Ascl - 10) * 10]
	.dis.mdF.rngVal configure -text "Distributions:  $mdSl"
    } else {
	set mdSl [expr ($Ascl - 19) * 100]
	.dis.mdF.rngVal configure -text "Distributions: $mdSl"
    }    
}

label .dis.mdF.rngVal -text "Distributions:  100" -bg #cccccc -fg black

label .dis.mdF.xAxVal -text "          10     100     1000" \
	-bg #cccccc -fg black

pack .dis.mdF.rngVal .dis.mdF.rng .dis.mdF.xAxVal -side top
 
radiobutton .dis.mdImp -variable MdImp -command {set MdCon 0} -state active \
	               -text "MD: Impulses"
radiobutton .dis.mdCon -variable MdCon -command {set MdImp 0} \
	               -text "MD: Contour"
.dis.mdImp select
.dis.mdCon deselect
set MdCon 0
grid .dis.mdImp -row 7 -column 1 
grid .dis.mdCon -row 7 -column 2

# Create Action Button

frame .frm -relief groove -borderwidth 3 -bg #cccccc
pack .frm -side left -expand y -fill both

button .frm.defB -text "Defaults" -command defP \
             -bg #cccccc -fg black 
pack .frm.defB -side top -expand y -fill both

proc defP {} {
    global PdisXl Alag DisFn MdisL
    set tmp [exec cat $DisFn | wc -l]
    set inter [split $tmp]
    set idx 0
    while {[lindex $inter $idx] == ""} {
        incr idx
    }
    set PdisXl [lindex $inter $idx]
    if {$PdisXl < 100} {
      set Alag $PdisXl
    } else {
      set Alag 100
    }
    set MdisL [exec cat $DisFn | sort -n | tail -1]
    .dis.mdF.rng set 20
}

button .frm.pB -text "Profile" -command disProP -bg #cccccc -fg black
pack .frm.pB -side top -expand y -fill both

proc disProP {} {
    global DisFn DisTl PdisXl
    if {$DisTl == ""} {
	set ProTl "Profile of $DisFn"
    } else {
	set ProTl $DisTl
    }

    if {$PdisXl == ""} {
	set ProXl [exec cat $DisFn | wc -l]
    } else {
	set ProXl $PdisXl
    }
    mkProGnu $DisFn $ProTl $ProXl
    exec gnuplot $DisFn.gnu
    exec ghostview $DisFn.ps &
}

proc mkProGnu {file title xLmt} {
    set wFn "$file.gnu"
    set wfp [open $wFn w]
    puts $wfp "set ylabel \"bits per Interval\" "
    puts $wfp "set xlabel \"Interval Number\" "
    puts $wfp "set nokey"
    puts $wfp "set terminal postscript"
    puts $wfp ""
    puts $wfp "set output \"$file.ps\" "
    puts $wfp "set title \"$title\" "
    puts $wfp "set xrange \[0:$xLmt\] "
    puts $wfp "set yrange \[0:\] "
    puts $wfp "plot '$file' with impulses"
    close $wfp
}

button .frm.aB -text "Autocorrelation" -command disAcP \
                   -bg #cccccc -fg black
pack .frm.aB -side top -expand y -fill both

proc disAcP {} {
    global DisFn DisTl Alag
    if {$DisTl == ""} {
	set AcTl "Autocorrelation of $DisFn"
    } else {
	set AcTl $DisTl
    }

    if {$Alag == ""} {
	set TLags [exec cat $DisFn | wc -l]
	if {$TLags < 100} {
	    set Lags $TLags
	} else {
	    set Lags 100
	}
    } else {
	set Lags $Alag
    }

    mkAc $DisFn "$DisFn.ac" 1 0 $Lags 
    mkAcGnu $DisFn $AcTl $Lags
    exec gnuplot $DisFn.ac.gnu
    exec ghostview $DisFn.ac.ps &
}

proc mkAcGnu {file title lag} {
    set wFn "$file.ac.gnu"
    set wfp [open $wFn w]
    puts $wfp "set ylabel \"Autocorrelation\" "
    puts $wfp "set xlabel \"Lag\" "
    puts $wfp "set nokey"
    puts $wfp "set terminal postscript"
    puts $wfp ""
    puts $wfp "set output \"$file.ac.ps\" "
    puts $wfp "set title \"$title\" "
    puts $wfp "plot \[0:$lag\] \[-1:1\] '$file.ac' with lines"
    close $wfp
}

button .frm.rsB -text "R/S Pox" -command disRSP -bg #cccccc -fg black
pack .frm.rsB -side top -expand y -fill both

proc disRSP {} {
    global DisFn DisTl
    if {$DisTl == ""} {
	set RSTl "R/S Pox Diagram of $DisFn"
    } else {
	set RSTl $DisTl
    }

    set blkSz 10
    mkRSdat $DisFn
    mkRS $blkSz $DisFn "$DisFn.rs" 
    mkRSGnu $DisFn $RSTl 
    exec gnuplot $DisFn.rs.gnu
    exec ghostview $DisFn.rs.ps &
}

proc mkRSdat {file} {
    set flSz [exec cat $file | wc -l]
    set wFn05 [open "rs.5.dat" w]
    set wFn10 [open "rs1.dat" w]

    puts $wFn10 "1 1"
    puts $wFn10 "$flSz $flSz"

    puts $wFn05 "1 1"
    if {$flSz >= 5} {puts $wFn05 "10 3.16"}
    if {$flSz >= 50} {puts $wFn05 "100 10"}
    if {$flSz >= 500} {puts $wFn05 "1000 31.6"}
    if {$flSz >= 5000} {puts $wFn05 "10000 100"}
    if {$flSz >= 50000} {puts $wFn05 "100000 316.2"}

    close $wFn10
    close $wFn05
}

proc mkRSGnu {file title} {
    set wFn "$file.rs.gnu"
    set wfp [open $wFn w]
    puts $wfp "set ylabel \"R/S Statistic\" "
    puts $wfp "set xlabel \"Sample Size\" "
    puts $wfp "set logscale x"
    puts $wfp "set logscale y"
    puts $wfp "set nokey"
    puts $wfp "set terminal postscript"
    puts $wfp ""
    puts $wfp "set xrange \[1:\] "
    puts $wfp "set output \"$file.rs.ps\" "
    puts $wfp "set title \"$title\" "
    puts $wfp "plot 'rs1.dat' with lines, \
                    'rs.5.dat' with lines, \
                    '$file.rs' with points"
    close $wfp
}

button .frm.varB -text "Variance" -command disVarP -bg #cccccc -fg black
pack .frm.varB -side top -expand y -fill both

proc disVarP {} {
    global DisFn DisTl
    if {$DisTl == ""} {
	set VarTl "Variance Time Plot of $DisFn"
    } else {
	set VarTl $DisTl
    }


    mkVar $DisFn "$DisFn.vr" 
    mkVarGnu $DisFn $VarTl 
    exec gnuplot $DisFn.vr.gnu
    exec ghostview $DisFn.vr.ps &
}

proc mkVarGnu {file title} {
    set wFn "$file.vr.gnu"
    set wfp [open $wFn w]
    puts $wfp "set ylabel \"Variance of Sample Mean\" "
    puts $wfp "set xlabel \"Aggregation Level\" "
    puts $wfp "set logscale x"
    puts $wfp "set logscale y"
    puts $wfp "set nokey"
    puts $wfp "set terminal postscript"
    puts $wfp ""
    puts $wfp "set output \"$file.vr.ps\" "
    puts $wfp "set title \"$title\" "
    puts $wfp "plot '$file.vr' with linespoints"
    close $wfp
}


button .frm.hisB -text "M.Distribution" -command disMDP \
                     -bg #cccccc -fg black
pack .frm.hisB -side top -expand y -fill both

proc disMDP {} {
    global DisFn DisTl mdSl MdisL MdCon
    if {$DisTl == ""} {
	set MdTl "Marginal Distribution of $DisFn"
    } else {
	set MdTl $DisTl
    }

    if {$MdisL == ""} {
	set MdisL [exec cat $DisFn | sort -n | tail -1]
    } 

    if {$MdCon == 0} {
	set sty "impulses"
    } else {
	set sty "lines"
    }

    mkHis $mdSl $MdisL $DisFn "$DisFn.md" 
    mkMDGnu $DisFn $MdTl $sty 
    exec gnuplot $DisFn.md.gnu
    exec ghostview $DisFn.md.ps &
}

proc mkMDGnu {file title sty} {
    set wFn "$file.md.gnu"
    set wfp [open $wFn w]
    puts $wfp "set ylabel \"Frequency (percentage)\" "
    puts $wfp "set xlabel \"Observed Value\" "
    puts $wfp "set nokey"
    puts $wfp "set terminal postscript"
    puts $wfp ""
    puts $wfp "set output \"$file.md.ps\" "
    puts $wfp "set title \"$title\" "
    puts $wfp "plot '$file.md' with $sty"
    close $wfp
}



##########################################################
#                                                        #
#                       KitAction Module                 #
#                                                        #
##########################################################


frame .kit -relief groove -borderwidth 3 -bg #cccccc
pack .kit -side left -expand y -fill both


button .kit.abtB -text "About" -command aboutP -bg #cccccc -fg black
pack .kit.abtB -expand y -fill both

proc aboutP {} {
    toplevel .kit.about -bg #cccccc
    wm title .kit.about "About SynTraff"
    label .kit.about.msg -wraplength 650 -justify center -bg #cccccc -fg black
    putInfo
    pack .kit.about.msg 
    frame .kit.about.spc -height 30 -bg #cccccc
    pack .kit.about.spc -fill x -expand y
    button .kit.about.ok -text "OK" -command wCloseP -bg #cccccc -fg black 
    pack .kit.about.ok 
}

proc wCloseP {} {
    destroy .kit.about
}

proc putInfo {} {
    .kit.about.msg configure -text \
"synTraff  (Version 3)  is  an  interactive  kit,\n\
to generate, refine, and analyse network traffic.\n\n\n\
\
Authors:\n\n\ 
Ram Balakrishnan and Carey Williamson\n\
Department of Computer Science\n\
University of Saskatchewan\n\
Saskatoon, Canada\n\n\n\
\
The authors would like to thank\n\
Carl Gutwin  and  Lidija Grahek\n\ 
for offering constructive\n\
suggestions throughout the project."
}


button .kit.extB -text "Exit" -command exit -bg #cccccc -fg black
pack .kit.extB -expand y -fill both















